/* ------------------------------------------------------------------------------------------------ */
/* FILE :		 Demo_1_TMX320_P28027_CCS.c															*/
/*																									*/
/* CONTENTS :																						*/
/*				Demo for the Olimex TMX320-P28027evaluation board (USB serial port)					*/
/*				Flashing info and control shown on the serial port									*/
/*              NOTE: THIS IS AN APPLICATION EXAMPLE WHERE THE HYBRID STACK CANNOT BE USED			*/
/*																									*/
/*				Serial port settings:																*/
/*							Baud Rate: 115200														*/
/*							Data bits:      8														*/
/*							Stop bits:      1														*/
/*							Parity:      none														*/
/*							Flow Ctrl:   none														*/
/*							Emulation: VT-100														*/
/*																									*/
/*																									*/
/* Copyright (c) 2014, Code-Time Technologies Inc. All rights reserved.								*/
/*																									*/
/* Code-Time Technologies retains all right, title, and interest in and to this work				*/
/*																									*/
/* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS							*/
/* OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF										*/
/* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL							*/
/* THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR								*/
/* OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,							*/
/* ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR							*/
/* OTHER DEALINGS IN THE SOFTWARE.																	*/
/*																									*/
/*																									*/
/*	$Revision: 1.1 $																				*/
/*	$Date: 2012/06/17 14:58:30 $																	*/
/*																									*/
/* ------------------------------------------------------------------------------------------------ */

#if OS_DEMO != 1
	#error "Wrong OS_DEMO value" 
#endif

#include "Abassi.h"

/* ------------------------------------------------ */

#include "DSP28x_Project.h"
#include <file.h>

#define BAUDRATE	   115200						/* Serial port baud rate						*/
#define SYS_FREQ	60000000L						/* Processor clock frequency					*/

int ___putchar(int);
int ___getchar(void);

void FctLed0(void);									/* Function attached to TaskLed0				*/
void FctLed1(void);									/* Function attached to TaskLed1				*/
void FctLed2(void);									/* Function attached to TaskLed2				*/
int  GetKey(void);
void IOinit(void);
void MemCopy(Uint16 *Src, Uint16 *End, Uint16 *Dst);
void UARTinit(void);
void Timer0ISR(void);								/* Timer 0 interupt handler						*/

/* ------------------------------------------------ */

SEM_t *SemAE;										/* Semaphore Adam & Eve blocks on				*/
SEM_t *SemLed0;										/* Sempahore TaskLed0 blocks on					*/
SEM_t *SemLed1;										/* Sempahore TaskLed1 blocks on					*/
SEM_t *SemLed2;										/* Sempahore TaskLed2 blocks on					*/

TSK_t *TaskLed0;									/* Task toggling LED 0							*/
TSK_t *TaskLed1;									/* Task toggling LED 1							*/
TSK_t *TaskLed2;									/* Task toggling LED 2							*/

static volatile int g_Rate;							/* Controils the LEDs toggling rate				*/

const char Running[] = "|/-\\";

char Flashing[] = "[ ]  [ ]  [ ]";

/* ------------------------------------------------------------------------------------------------ */
/* ------------------------------------------------------------------------------------------------ */

main()
{
int ii;												/* General purpose								*/
int jj;												/* General purpose								*/


/* ------------------------------------------------ */
/* 	Processor clock set-up							*/

	InitSysCtrl();
	MemCopy(&RamfuncsLoadStart, &RamfuncsLoadEnd, &RamfuncsRunStart);
	InitFlash();
	InitSciaGpio();
	InitCpuTimers();

/* ------------------------------------------------ */
/* UART set-up										*/

	UARTinit();
	IOinit();

/* ------------------------------------------------ */
/* 	Start the RTOS									*/

	OSstart();

/* ------------------------------------------------ */
/* SysTick Set-up (Timer 2)							*/

	OSmapISR(14, 0, TIMtick);						/* Use the RTOS timer for Abassi, vect ID 14	*/

	ConfigCpuTimer(&CpuTimer2, 1.0,(float)(((SYS_FREQ/100000)*(OS_TIMER_US))/(10)));
	CpuTimer2Regs.TCR.all         = 0x4001;
	IER                          |= M_INT14;		/* Timer 2 is attached to INT #14				*/

/* ------------------------------------------------ */
/* Timer 0 Set-up									*/

	OSmapISR(38, 1, &Timer0ISR);					/* Timer 0 is vector ID 38						*/

	ConfigCpuTimer(&CpuTimer0, 1.0, 0.81*(float)(((SYS_FREQ/100000)*(OS_TIMER_US))/(10)));
	CpuTimer0Regs.TCR.all         = 0x4001;
	IER                          |= M_INT1;			/* Timer 0 is attached to PIE interrupt group 1	*/

	PieCtrlRegs.PIEIER1.bit.INTx7 = 1;				/* Enable timer 0 which is attached to INT1.7	*/

/* ------------------------------------------------ */
/* Application set-up								*/

	TaskLed0 = TSKcreate("LED 0", 1, TC_TSK_STACK, &FctLed0, 1);
	TaskLed1 = TSKcreate("LED 1", 1, TC_TSK_STACK, &FctLed1, 1);
	TaskLed2 = TSKcreate("LED 2", 1, TC_TSK_STACK, &FctLed2, 1);

	SemAE    = SEMopen("A&E");
	SemLed0  = SEMopen("LED 0");
	SemLed1  = SEMopen("LED 1");
	SemLed2  = SEMopen("LED 2");

/* ------------------------------------------------ */

	puts("\n\n");									/* Welcome banner								*/
	puts("\n           Abassi Demo 1\n");
	puts("\n        Olimex TMX320-28027\n");
	puts("\n      Use + - on serial port\n\n");

/* ------------------------------------------------ */
/* Interrupts ON, the application runs				*/

	g_Rate = 5;

	OSeint(1);
	ERTM;

 	TSKsetPrio(TSKgetID(""), OS_PRIO_MIN);			/* No IdleTask created, Adam-Eve is Idle Task	*/

	jj = 0;											/* From here on, the code is the Idle Task		*/
   	for(;;) {										/* Check key to be pressed to change the rate	*/
		switch(GetKey()) {
		case '+':									/* Key '+' pressed, increase the toggling rate	*/
			ii = g_Rate - 1;						/* which means to decrease the variable g_Rate	*/
			if (ii <= 0) {							/* Floor g_Rate at 0							*/
				ii = 1;
			}
			g_Rate = ii;
			break;
		case '-':									/* Key '-' pressed, decrease the toggling rate	*/
			ii = g_Rate + 1;						/* which means to increase the variable g_Rate	*/
			if (ii > 10) {							/* Ceiling g_Rate at 10							*/
				ii = 10;
			}
			g_Rate = ii;;
			break;
		default:									/* Any other key is ignored						*/
			break;
		}

		ii = g_Rate;								/* Update the info on the serial port			*/
		printf("%c  Current rate %d%s  %s\r", Running[jj++], ii, (ii<10)?" ":"", Flashing);

		jj &= 3;
	}
}

/* ------------------------------------------------------------------------------------------------ */
/*																									*/
/* Idle Task																						*/
/*																									*/
/* ------------------------------------------------------------------------------------------------ */

void IdleTask(void)
{
	TSKselfSusp();									/* A&E is set to Idle priority so we can		*/
													/* disappear									*/
	for(;;);
}

/* ------------------------------------------------------------------------------------------------ */
/* 																									*/
/* LED0 task																						*/
/* 																									*/
/* ------------------------------------------------------------------------------------------------ */

void FctLed0(void)
{
	for ( ; ; ) {
		SEMwaitBin(SemLed0, -1);					/* Wait for ISR to post SemLed0					*/
		SEMpost(SemLed1);							/* Inform TaskLed1 to toggle LED 1				*/
		Flashing[1] = '*';							/* Show on serial port LED 0 on					*/

		SEMwaitBin(SemLed0, -1);					/* Wait for ISR to post SemLed0					*/
		SEMpost(SemLed1);							/* Inform TaskLed1 to toggle LED 1				*/
		Flashing[1] = ' ';							/* Show on serial port LED 0 off				*/
	}
}

/* ------------------------------------------------------------------------------------------------ */
/* 																									*/
/* LED1 task																						*/
/* 																									*/
/* ------------------------------------------------------------------------------------------------ */

void FctLed1(void)
{
	for ( ; ; ) {
		SEMwaitBin(SemLed1, -1);					/* Each posting of SemLed1 toggle LED 1			*/
		Flashing[6] = '*';

		SEMwaitBin(SemLed1, -1);					/* Each posting of SemLed1 toggle LED 1			*/
		Flashing[6] = ' ';
	}
}

/* ------------------------------------------------------------------------------------------------ */
/* 																									*/
/* LED2 task																					*/
/* 																									*/
/* ------------------------------------------------------------------------------------------------ */

void FctLed2(void)
{
	for ( ; ; ) {
		SEMwaitBin(SemLed2, -1);					/* Wait for ISR to post SemLed2					*/
		SEMpost(SemLed1);							/* Inform TaskLed1 to toggle LED 1				*/
		Flashing[11] = '*';							/* Show on serial port LED 2 on					*/

		SEMwaitBin(SemLed2, -1);					/* Wait for ISR to post SemLed2					*/
		SEMpost(SemLed1);							/* Inform TaskLed1 to toggle LED 1				*/
		Flashing[11] = ' ';							/* Show on serial port LED 2 off				*/
	}
}

/* ------------------------------------------------------------------------------------------------ */
/*																									*/
/* Timer call back function																			*/
/* 																									*/
/* ------------------------------------------------------------------------------------------------ */

void TIMcallBack(void)
{
static int AEcount=0;								/* Counter to control when to awake A&E			*/
static int Count=0;									/* Counter to control when to toggle LED 0		*/


	if (++Count >= g_Rate*4*(OS_TICK_PER_SEC/10)) {	/* Determine if it's time to toggle LED 0		*/
		SEMpost(SemLed0);							/* When it is, post the semaphore TaslLed0 is	*/
		Count = 0;									/* blocked on									*/
	}
	
	if (++AEcount >= (OS_TICK_PER_SEC/10)) {		/* Determine if it's time to awake A&E			*/
		SEMpost(SemAE);								/* When it is, post the semaphoreA&E is blocked	*/
		AEcount = 0;								/* on											*/
	}

	return;
}

/* ------------------------------------------------------------------------------------------------ */
/* 																									*/
/* Timer 0 interrupt function																		*/
/*																									*/
/* All this function does it to post the SemLed2 semaphore											*/
/* 																									*/
/* ------------------------------------------------------------------------------------------------ */

void Timer0ISR(void)
{
static int Count=0;

	IER |= M_INT1;									/* Re-enable INT1								*/
	PieCtrlRegs.PIEACK.bit.ACK1 = 0x1;				/* Timer #0 is in PIE group 1, re-enable it		*/

	if (++Count >= g_Rate*4*(OS_TICK_PER_SEC/10)) {	/* Determine if it's time to toggle LED 2		*/
		SEMpost(SemLed2);							/* When it is, post the semaphore TaslLed2 is	*/
		Count = 0;									/* blocked on									*/
	}

	return;
}

/* ------------------------------------------------------------------------------------------------ */

void MemCopy(Uint16 *Src, Uint16 *End, Uint16 *Dst)
{
	while (Src < End) {
		*Dst++ = *Src++;
	}
	return;
}

/* ------------------------------------------------------------------------------------------------ */
/*																									*/
/* UART functions needed by the Peripheral library to access the UART								*/
/*																									*/
/* ------------------------------------------------------------------------------------------------ */

void UARTinit(void)
{

    SciaRegs.SCICCR.all             = 0x0007;		/* 1 stop bit,  No loopback, No parity,8 bits,	*/
													/* async mode, idle-line protocol				*/
    SciaRegs.SCICTL1.all            = 0x0003;		/* enable TX, RX, internal SCICLK,				*/
													/* Disable RX ERR, SLEEP, TXWAKE				*/
    SciaRegs.SCICTL2.all            = 0x0000;
    SciaRegs.SCICTL2.bit.TXINTENA   = 0;			/* No TX interrupt								*/
    SciaRegs.SCICTL2.bit.RXBKINTENA = 0;			/* No RX interrupt								*/
	SciaRegs.SCIHBAUD               = 0x0000;
	SciaRegs.SCILBAUD               = ((SYS_FREQ)/(BAUDRATE))/32;

    SciaRegs.SCICTL1.all            = 0x0023;		/* Relinquish SCI from Reset					*/		

    SciaRegs.SCIFFTX.all            = 0xE040;		/* Set-up FIFOs									*/
    SciaRegs.SCIFFRX.all            = 0x2044;
    SciaRegs.SCIFFCT.all            = 0x0;

	return;
}

/* ------------------------------------------------------------------------------------------------ */

int UARTopen(const char *path, unsigned int flags, int fd)
{
	return(0);
}

/* ------------------------------------------------------------------------------------------------ */

int UARTclose(int fd)
{
	return(0);
}

/* ------------------------------------------------------------------------------------------------ */

int UARTread(int fd, char *buf, unsigned int count)
{
	*buf = ___getchar();
	return(1);
}

/* ------------------------------------------------------------------------------------------------ */

int UARTwrite(int fd, const char *buf, unsigned int count)
{
int ii;

	for(ii=0 ; ii<count ; ii++) {
		 ___putchar(*buf++);
	}
	return(count);
}

/* ------------------------------------------------------------------------------------------------ */

off_t UARTseek(int fd, off_t off, int start)
{
	return(0);
}

/* ------------------------------------------------------------------------------------------------ */

int UARTunlink(const char *path)
{
	return(0);
}

/* ------------------------------------------------------------------------------------------------ */

int UARTrename(const char *old, const char *new)
{
	return(0);
}

/* ------------------------------------------------------------------------------------------------ */

void IOinit(void) {
	add_device("uart", _MSA, UARTopen, UARTclose, UARTread, UARTwrite, UARTseek, UARTunlink, UARTrename);
	freopen("uart:stdout", "w", stdout);
	setvbuf(stdout, NULL, _IONBF, 0);
	freopen("uart:stdin", "r", stdin);
	setvbuf(stdin, NULL, _IONBF, 0);
}

/* ------------------------------------------------------------------------------------------------ */

int ___putchar(int ccc)
{
	if (ccc == '\n') {
    	for( ; SciaRegs.SCIFFTX.bit.TXFFST != 0 ; );
	    SciaRegs.SCITXBUF = '\r';
	}
   	for( ; SciaRegs.SCIFFTX.bit.TXFFST != 0 ; );
    SciaRegs.SCITXBUF = ccc;

	return(ccc);
}

/* ------------------------------------------------------------------------------------------------ */

int ___getchar(void)
{
int RetVal;

	for ( ; SciaRegs.SCIFFRX.bit.RXFFST == 0 ; );
	RetVal = SciaRegs.SCIRXBUF.all;

	return(RetVal);
}

/* ------------------------------------------------------------------------------------------------ */

int GetKey(void)
{
int RetVal;

	RetVal = 0;

	if (SciaRegs.SCIFFRX.bit.RXFFST != 0) {
		RetVal = SciaRegs.SCIRXBUF.all;
	}

	return(RetVal);
}

/* EOF */
